@@ -14,12 +14,9 @@ module OmniAuth
       BASE_SCOPES = %w[profile email openid].freeze
       DEFAULT_SCOPE = 'email,profile'
       USER_INFO_URL = 'https://www.googleapis.com/oauth2/v3/userinfo'
-      IMAGE_SIZE_REGEXP = /(s\d+(-c)?)|(w\d+-h\d+(-c)?)|(w\d+(-c)?)|(h\d+(-c)?)|c/
-      AUTHORIZE_OPTIONS = %i[access_type hd login_hint prompt request_visible_actions scope state redirect_uri include_granted_scopes openid_realm device_id device_name]
+      AUTHORIZE_OPTIONS = %i[access_type hd login_hint prompt request_visible_actions scope state redirect_uri include_granted_scopes enable_granular_consent openid_realm device_id device_name]
 
       option :name, 'google_oauth2'
-      option :skip_friends, true
-      option :skip_image_info, true
       option :skip_jwt, false
       option :jwt_leeway, 60
       option :authorize_options, AUTHORIZE_OPTIONS
@@ -76,17 +73,11 @@ module OmniAuth
 
           # We have to manually verify the claims because the third parameter to
           # JWT.decode is false since no verification key is provided.
-          ::JWT::Verify.verify_claims(decoded,
-                                      verify_iss: true,
-                                      iss: ALLOWED_ISSUERS,
-                                      verify_aud: true,
-                                      aud: options.client_id,
-                                      verify_sub: false,
-                                      verify_expiration: true,
-                                      verify_not_before: true,
-                                      verify_iat: false,
-                                      verify_jti: false,
-                                      leeway: options[:jwt_leeway])
+          ::JWT::Claims.verify_payload!(decoded,
+                                        iss: ALLOWED_ISSUERS,
+                                        aud: options.client_id,
+                                        exp: { leeway: options.jwt_leeway },
+                                        nbf: { leeway: options.jwt_leeway })
 
           hash[:id_info] = decoded
         end
@@ -155,7 +146,7 @@ module OmniAuth
 
       def get_scope(params)
         raw_scope = params[:scope] || DEFAULT_SCOPE
-        scope_list = raw_scope.split(' ').map { |item| item.split(',') }.flatten
+        scope_list = raw_scope.split.map { |item| item.split(',') }.flatten
         scope_list.map! { |s| s =~ %r{^https?://} || BASE_SCOPES.include?(s) ? s : "#{BASE_SCOPE_URL}#{s}" }
         scope_list.join(' ')
       end
@@ -178,17 +169,16 @@ module OmniAuth
       def image_url
         return nil unless raw_info['picture']
 
-        u = URI.parse(raw_info['picture'].gsub('https:https', 'https'))
+        u = URI.parse(raw_info['picture'])
 
-        path_index = u.path.to_s.index('/photo.jpg')
+        md = u.path.to_s.match(/(.*)(=((w[0-9]*|h[0-9]*|s[0-9]*|c|p)-?)*)$/)
 
-        if path_index && image_size_opts_passed?
-          u.path.insert(path_index, image_params)
+        # Check for sizing, remove if present.
+        u.path = md[1] if md && !md[1].nil? && !md[2].nil?
+
+        if image_size_opts_passed?
+          u.path += image_params
           u.path = u.path.gsub('//', '/')
-
-          # Check if the image is already sized!
-          split_path = u.path.split('/')
-          u.path = u.path.sub("/#{split_path[-3]}", '') if split_path[-3] =~ IMAGE_SIZE_REGEXP
         end
 
         u.query = strip_unnecessary_query_parameters(u.query)
@@ -202,23 +192,25 @@ module OmniAuth
 
       def image_params
         image_params = []
-        if options[:image_size].is_a?(Integer)
+        case options[:image_size]
+        when Integer
           image_params << "s#{options[:image_size]}"
-        elsif options[:image_size].is_a?(Hash)
+        when Hash
           image_params << "w#{options[:image_size][:width]}" if options[:image_size][:width]
           image_params << "h#{options[:image_size][:height]}" if options[:image_size][:height]
         end
         image_params << 'c' if options[:image_aspect_ratio] == 'square'
+        image_params << 'p' if options[:image_aspect_ratio] == 'smart'
 
-        '/' + image_params.join('-')
+        "=#{image_params.join('-')}"
       end
 
       def strip_unnecessary_query_parameters(query_parameters)
         # strip `sz` parameter (defaults to sz=50) which overrides `image_size` options
         return nil if query_parameters.nil?
 
-        params = CGI.parse(query_parameters)
-        stripped_params = params.delete_if { |key| key == 'sz' }
+        params = URI.decode_www_form(query_parameters)
+        stripped_params = params.delete_if { |key, _value| key == 'sz' }
 
         # don't return an empty Hash since that would result
         # in URLs with a trailing ? character: http://image.url?
@@ -231,7 +223,7 @@ module OmniAuth
         return nil unless access_token
 
         @token_info ||= Hash.new do |h, k|
-          h[k] = client.request(:get, 'https://www.googleapis.com/oauth2/v3/tokeninfo', params: { access_token: access_token }).parsed
+          h[k] = client.request(:post, 'https://www.googleapis.com/oauth2/v3/tokeninfo', body: { access_token: access_token }).parsed
         end
 
         @token_info[access_token]
